import logging
from abc import ABC
from dataclasses import dataclass
from dacite import from_dict, Config, UnexpectedDataError, UnionMatchError
from typing import Optional, Union, Any
import json
import yaml
from os import path, sep
from pathlib import Path

"""
Collection of dataclasses related to the configuration of the application.

"""
logger = logging.getLogger(__name__)


@dataclass
class BaseAuthConfig:

    def getAuthType(self) -> str:
        raise NotImplementedError("You must implement this method to return the String type of the auth")


@dataclass
class ApiKeyAuthConfig(BaseAuthConfig):
    api_key: str

    def getAuthType(self) -> str:
        return "ApiKey"


@dataclass
class OAuth2AuthConfig(BaseAuthConfig):
    auth_url: Optional[str]
    app_client_id: str
    app_client_secret: str

    def getAuthType(self) -> str:
        return "OAuth2"

    def validate(self):
        pass


@dataclass
class ClientConfig:
    """
    Wrapper for all configuration needed to work with Vectara.
    """

    customer_id: str
    auth: Union[ApiKeyAuthConfig, OAuth2AuthConfig]

    def validate(self) -> [str]:
        errors = []

        if not self.customer_id:
            errors.append(f"In [{__class__.__name__}] You must define the field [customer_id]")

        return errors


def _tryCreateAuth(data_class, input):
    try:
        from_dict(data_class=data_class, data=input, config=Config(strict=True))
        return True, None
    except Exception as e:
        return False, str(e)


def loadConfig(config: str) -> ClientConfig:
    """
    Loads our configuration from JSON onto our data classes.

    :param config: the input configuration in JSON format.
    :return: the parsed client configuration1
    :raises TypeError: if the configuration cannot be parsed correctly
    """
    logger.info(f"Loading config from {config}")

    try:
        config_dict = json.loads(config)



        return from_dict(ClientConfig, config_dict, config=Config(strict=True))
    except UnionMatchError as e:
        raise TypeError(e)
    except UnexpectedDataError as e:
        raise TypeError(e)

class BaseConfigLoader(ABC):
    CONFIG_FILE_NAME = ".vec_auth.yaml"

    DEFAULT_CONFIG_NAME = "default"

    def __init__(self, profile: str = None):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.profile = profile

    def load(self):
        """
        :return: The client configuration in our domain class ClientConfig
        :raises TypeError: Should be implemented in subclasses.
        """
        raise Exception("Define in sublcass")

    def _convert_dict_config(self, config_dict: dict[str:Any]) -> ClientConfig:
        """
        Helper method for all subclasses of BaseConfigLoader to parse a dict into our config domain classes.

        :param config_dict: the input configuration as a dict
        :return: the parsed client configuration
        :raises TypeError: if the configuration cannot be parsed correctly
        """

        # First we're going to manually try each Authentication Type for better logging.
        # This is because the dicate library just gives the user a generic error
        # without any logging.
        auth = config_dict["auth"]
        if (auth):
            oauth2_success, oauth2_error_msg = _tryCreateAuth(OAuth2AuthConfig, auth)
            api_key_success, api_key_error_msg = _tryCreateAuth(ApiKeyAuthConfig, auth)

            if not oauth2_success and not api_key_success:
                logger.error(f"Invalid Authentication Configuration:\n{json.dumps(auth, indent=4)}")
                logger.error(f"Unable to cast auth to OAuth2 configuration block: {oauth2_error_msg}")
                logger.error(f"Unable to cast auth to API Key configuration block: {api_key_error_msg}")

                raise TypeError(
                    f"Could not use polymorphism to cast auth to either OAuth2 or API Key config, see errors above in log.")

        self.logger.info("Parsing config")
        try:
            client_config = from_dict(data_class=ClientConfig, data=config_dict, config=Config(strict=True))
            client_config.validate()
            return client_config
        except UnexpectedDataError as e:
            raise TypeError(f"Unable to build configuration: {e}") from None

    def _load_yaml_config(self, final_config_path):
        with open(final_config_path, 'r') as yaml_stream:
            creds = yaml.safe_load(yaml_stream)

            if self.profile:
                self.logger.info(f"Loading specified profile [{self.profile}]")
                profile_to_load = self.profile
            else:
                self.logger.info(f"Loading default configuration [{BaseConfigLoader.DEFAULT_CONFIG_NAME}]")
                profile_to_load = BaseConfigLoader.DEFAULT_CONFIG_NAME

            if profile_to_load in creds:
                return creds[profile_to_load]
            else:
                raise TypeError(f"Specified profile [{profile_to_load}] not found in [{final_config_path}]")


class JsonConfigLoader(BaseConfigLoader):
    """
    Loads our configuration from JSON
    """

    def __init__(self, config_json: str, profile: str = None):
        super().__init__(profile=profile)
        self.config_json = config_json

    def load(self):
        self.logger.info("Loading configuration from JSON string")
        config_dict = json.loads(self.config_json)
        return self._convert_dict_config(config_dict)


class PathConfigLoader(BaseConfigLoader):
    """
    Loads our configuration from the specified folder/file
    """

    def __init__(self, config_path, profile: str = None):
        super().__init__(profile=profile)
        self.config_path = config_path

    def load(self):
        self.logger.info(f"Loading configuration from path {self.config_path}")

        if path.exists and path.isdir(self.config_path):
            self.logger.info(f"Configuration param is a path, looking for {BaseConfigLoader.CONFIG_FILE_NAME}")
            looking_for = self.config_path + sep + BaseConfigLoader.CONFIG_FILE_NAME
            if not path.exists(looking_for) or not path.isfile(looking_for):
                raise TypeError(f"Unable to find configuration file [{BaseConfigLoader.CONFIG_FILE_NAME}]"
                                f" within specified directory [{self.config_path}]")
        elif path.exists and path.isfile(self.config_path):
            self.logger.info(f"Configuration param is a file")
            looking_for = self.config_path
        else:
            raise TypeError(f"Path [{self.config_path}] does not exist.")

        config_dict = self._load_yaml_config(looking_for)
        return self._convert_dict_config(config_dict)

# TODO Finish implementing this before commit
class HomeConfigLoader(BaseConfigLoader):
    """
    Loads our configuration from the users home directory
    """

    def __init__(self, profile: str = None):
        super().__init__(profile=profile)

    def load(self):
        home = str(Path.home())
        self.logger.info(f"Loading configuration from users home directory [{home}]")

        looking_for = home + sep + BaseConfigLoader.CONFIG_FILE_NAME
        if not path.exists(looking_for) or not path.isfile(looking_for):
            raise TypeError(f"Unable to find configuration file [{BaseConfigLoader.CONFIG_FILE_NAME}]"
                            f" within home directory [{home}]")
        config_dict = self._load_yaml_config(looking_for)
        return self._convert_dict_config(config_dict)





from abc import ABC
from typing import Type, TypeVar, List
import logging
import threading
import json





class BaseFormatter(ABC):

    def __init__(self):
        pass

    def heading(self, heading: str, level: int = 1):
        raise Exception("Implement in subclass")

    def sentence(self, sentence: str):
        raise Exception("Implement in subclass")

    def link(self, text: str, url: str):
        raise Exception("Implement in subclass")

    def paragraph(self, paragraph: str):
        raise Exception("Implement in subclass")

    def bold(self, text: str):
        raise Exception("Implement in subclass")

    def italic(self, text: str):
        raise Exception("Implement in subclass")

    def list(self, items: List[str], level: int = 1):
        raise Exception("Implement in subclass")

    def rtl(self, text: str):
        raise Exception("Implement in subclass")

    def code(self, text: str, language=None):
        raise Exception("Implement in subclass")

class MarkdownFormatter(BaseFormatter):

    def __init__(self):
        pass

    def heading(self, heading: str, level: int = 1):
        indent = "#" * level
        return f'\n{indent} {heading}'

    def sentence(self, sentence: str):
        return sentence

    def link(self, text, url):
        return f"[{text}]({url})"

    def paragraph(self, paragraph: str):
        return f"\n\n{paragraph}\n"

    def bold(self, text: str):
        return f"**{text}**"

    def italic(self, text: str):
        return f"*{text}*"

    def list(self, items: List[str], level: int = 1):
        if level < 1:
            raise TypeError("List level must be greater than 0")
        indent = " " * (level - 1)
        results = [f"{indent} {idx + 1}. {item}" for idx, item in enumerate(items)]
        return "\n" + "\n".join(results) + "\n"

    def rtl(self, text):
        return '<div dir="rtl">\n' + text + '</div>'

    def code(self, text: str, language=None):
        result = "\n```"
        if language:
            result += language
        result += str(text)
        result += "\n```\n"
        return result





prompt_text = (
    '[ {"role": "system", "content": "You are a human resources manager who takes the search results and summarizes them as a coherent response. Only use information provided in this chat. Respond in the language denoted by ISO 639 code \\"$vectaraLangCode\\"."}, \n'  # ,\n'
    '#foreach ($qResult in $vectaraQueryResults) \n'
    '   #if ($foreach.first) \n'
    '   {"role": "user", "content": "Search for \\"$esc.java(${vectaraQuery})\\", and give me the first search result."}, \n'
    '   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n'
    '   #else \n'
    '   {"role": "user", "content": "Give me the \\"$vectaraIdxWord[$foreach.index]\\" search result."}, \n'
    '   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n'
    '   #end \n'
    ' #end \n'
    '{"role": "user", "content": "Generate a detailed answer (that is no more than 300 words) for the query \\"$esc.java(${vectaraQuery})\\" solely based on the search results in this chat. You must only use information from the provided results. Cite search results using \\"[number]\\" notation. Only cite the most relevant results that answer the question accurately." } ]')


class BasePromptFactory(ABC):

    def __init__(self):
        pass

    def build(self):
        raise Exception("Implement in subclasses")


class SimplePromptFactory(BasePromptFactory):
    SYSTEM_PROMPT_TEMPLATE = 'You are a {persona} who takes the search results and {a_cite_text} {just_rag_text} Respond in the language denoted by ISO 639 code \\"$vectaraLangCode\\".'
    USER_PROMPT_TEMPLATE = 'Generate a detailed answer (that is no more than {max_word_count} words) for the query \\"$esc.java(${{vectaraQuery}})\\" {just_rag_text} {b_cite_text}'

    A_DO_CITE = "summarizes them as a coherent response,"
    A_DO_NOT_CITE = "only return the most relevant answer. Do not iterate over each question,"

    B_DO_CITE = 'Cite search results using \\\\\\"[number]\\\\\\" notation. Only cite the most relevant results that answer the question accurately.'
    B_DO_NOT_CITE = 'Do not cite search results.'

    RAG_ONLY = 'solely based on the search results in this chat. You must only use information from the provided results.'
    ALLOW_NON_RAG = 'preferably based on the search results in this chat. You may allow additional information you know in the results.'

    """
    Generates a valid prompt text

    """

    def __init__(self, persona: str, max_word_count: int = 300, cite: bool = True, just_rag=True):
        super().__init__()
        self.persona = persona
        self.max_word_count = max_word_count
        self.cite = cite
        self.just_rag = just_rag

    def build(self):
        lines = []

        if self.just_rag:
            just_rag_text = SimplePromptFactory.RAG_ONLY
        else:
            just_rag_text = SimplePromptFactory.ALLOW_NON_RAG

        if self.cite:
            a_cite_text = SimplePromptFactory.A_DO_CITE
            b_cite_text = SimplePromptFactory.B_DO_CITE
        else:
            a_cite_text = SimplePromptFactory.A_DO_NOT_CITE
            b_cite_text = SimplePromptFactory.B_DO_NOT_CITE

        system_prompt = SimplePromptFactory.SYSTEM_PROMPT_TEMPLATE.format(
            persona=self.persona, just_rag_text=just_rag_text, a_cite_text=a_cite_text
        )

        user_prompt = SimplePromptFactory.USER_PROMPT_TEMPLATE.format(
            max_word_count=self.max_word_count, just_rag_text=just_rag_text, b_cite_text=b_cite_text
        )

        # Append the system indicator.
        lines.append(f'[ {{"role": "system", "content": "{system_prompt}"}}, \n')

        # Append the pre-requisite 'for-loop' for RAG.
        lines.append('#foreach ($qResult in $vectaraQueryResults) \n')
        lines.append('   #if ($foreach.first) \n')
        lines.append(
            '   {"role": "user", "content": "Search for \\"$esc.java(${vectaraQuery})\\", and give me the first search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #else \n')
        lines.append(
            '   {"role": "user", "content": "Give me the \\"$vectaraIdxWord[$foreach.index]\\" search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #end \n')
        lines.append(' #end \n')

        # Append the Final user phrasing.
        # TODO interpolate.
        lines.append(f'{{"role": "user", "content": "{user_prompt}" }} ]')

        return "".join(lines)


class StandardPromptFactory(BasePromptFactory):
    """
    Generates a valid prompt text

    """

    def __init__(self, system_prompt=None, user_prompt=None):
        super().__init__()
        self.system_prompt = system_prompt
        self.user_prompt = user_prompt

    def build(self):
        lines = []
        # Append the system indicator.
        lines.append(f'[ {{"role": "system", "content": "{self.system_prompt}"}}, \n')

        # Append the pre-requisite 'for-loop' for RAG.
        lines.append('#foreach ($qResult in $vectaraQueryResults) \n')
        lines.append('   #if ($foreach.first) \n')
        lines.append(
            '   {"role": "user", "content": "Search for \\"$esc.java(${vectaraQuery})\\", and give me the first search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #else \n')
        lines.append(
            '   {"role": "user", "content": "Give me the \\"$vectaraIdxWord[$foreach.index]\\" search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #end \n')
        lines.append(' #end \n')

        # Append the Final user phrasing.
        # TODO interpolate.
        lines.append(f'{{"role": "user", "content": "{self.user_prompt}" }} ]')

        return "".join(lines)


class ChatPromptFactory(BasePromptFactory):
    SYSTEM_PROMPT_TEMPLATE = 'You are a {chat_persona} talking with a customer, respond to small talk in a nice way. You must not say you are an AI model. Provide a short answer from the search results, though you can go into more detail if requested from the user. Do not iterate over each question, just provide a short answer based on prior assistant answers in this chat. You may allow additional information you know in the results if nothing relevant is found. Respond in the language denoted by ISO 639 code \\"$vectaraLangCode\\".'
    USER_PROMPT_TEMPLATE = 'Generate a chat response which is part of a back-and-forth, that is no more than {max_word_count} words, for the query \\"$esc.java(${{vectaraQuery}})\\" preferably based on the interactions in this chat. Please ask for more information to help clarify if needed. If the response answers the question, please finish with a closing phrase that uses \\"does that answer your question\\" to confirm resolution.'

    def __init__(self, chat_persona="Customer Support", name="Gary", max_word_count=300, prompt_metadata={}):
        super().__init__()
        self.logger = logging.getLogger(self.__class__.__name__)
        self.chat_persona = chat_persona
        self.name = name
        self.messages = []
        self.max_word_count = max_word_count
        self.prompt_metadata = prompt_metadata

    def add_user_message(self, user_message):
        self.messages.append({"role": "user", "content": user_message})

    def add_assistant_message(self, assistant_message):
        self.messages.append({"role": "assistant", "content": assistant_message})

    def add_user_assistant_pair(self, user_message, assistant_message):
        self.messages.append({"role": "user", "content": user_message})
        self.messages.append({"role": "assistant", "content": assistant_message})

    def build(self):
        system_prompt = ChatPromptFactory.SYSTEM_PROMPT_TEMPLATE.format(
            chat_persona=self.chat_persona
        )

        user_prompt = ChatPromptFactory.USER_PROMPT_TEMPLATE.format(
            max_word_count=self.max_word_count
        )

        lines = []

        # Append the system indicator.
        lines.append(f'[ {{"role": "system", "content": "{system_prompt}"}}, \n')

        # Insert prior context user/assistant messages.
        for message in self.messages:
            lines.append(f'    {json.dumps(message)}, \n')
        # Append the pre-requisite 'for-loop' for RAG.
        lines.append('#foreach ($qResult in $vectaraQueryResults) \n')

        metadata = ""
        if (self.prompt_metadata):
            bits = []
            for key in self.prompt_metadata.keys():
                value = self.prompt_metadata[key]
                bits.append(f"{key}: ")

        # Insert the Retrieval results.
        lines.append('   #if ($foreach.first) \n')
        lines.append(
            '   {"role": "user", "content": "Search for \\"$esc.java(${vectaraQuery})\\", and give me the first search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #else \n')
        lines.append(
            '   {"role": "user", "content": "Give me the \\"$vectaraIdxWord[$foreach.index]\\" search result."}, \n')
        lines.append('   {"role": "assistant", "content": "$esc.java(${qResult.getText()})" }, \n')
        lines.append('   #end \n')
        lines.append(' #end \n')

        # Append the Final user phrasing.
        lines.append(f'{{"role": "user", "content": "{user_prompt}" }} ]')

        result = "".join(lines)
        self.logger.info("Chat prompt is:\n" + result)
        return result


class CountDownLatch:
    def __init__(self, count=1):
        self.count = count
        self.lock = threading.Condition()

    def count_down(self):
        self.lock.acquire()
        self.count -= 1
        if self.count <= 0:
            self.lock.notify_all()
        self.lock.release()

    def sweat_it_out(self):
        """
        Because await is a keyword.

        :return:
        """
        self.lock.acquire()
        while self.count > 0:
            self.lock.wait()
        self.lock.release()





import logging
import vectara_client
from vectara_client.config import JsonConfigLoader, PathConfigLoader, HomeConfigLoader
from vectara_client.auth import OAuthUtil, ApiKeyUtil
from vectara_client.api import (ChatsApi, CorporaApi, DocumentsApi, EncodersApi, IndexApi, JobsApi,
                                LargeLanguageModelsApi, QueriesApi, RerankersApi, UploadApi, UsersApi)
# from vectara_client.corpus import CorpusManager
from vectara_client.models.app_client import AppClient
from vectara_client.models.create_app_client_request import CreateAppClientRequest
from vectara_client.rest import ApiException
from pprint import pprint
import os


class Client:

    def __init__(self, customer_id: str, chats_api: ChatsApi, corpora_api: CorporaApi, documents_api: DocumentsApi,
                 encoders_api: EncodersApi, index_api: IndexApi, jobs_api: JobsApi,
                 llms_api: LargeLanguageModelsApi, queries_api: QueriesApi, rerankers_api: RerankersApi,
                 upload_api: UploadApi, users_api: UsersApi):
        self.logging = logging.getLogger(self.__class__.__name__)
        logging.info("initializing Client")
        self.customer_id = customer_id

        self.chats_api = chats_api
        self.corpora_api = corpora_api
        self.documents_api = documents_api
        self.encoders_api = encoders_api
        self.index_api = index_api
        self.jobs_api = jobs_api
        self.llms_api = llms_api
        self.queries_api = queries_api
        self.rerankers_api = rerankers_api
        self.upload_api = upload_api
        self.users_api = users_api


class Factory():

    def __init__(self, config_path: str = None, config_json: str = None, profile: str = None):
        """
        Initialize our factory using configuration which may either be in a file or serialized in a JSON string

        :param config_path: the file containing our configuration
        :param config_json: the JSON containing our configuration
        """

        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.info("initializing builder")
        self.config_path = config_path
        self.config_json = config_json
        self.profile = profile

    def build(self) -> Client:
        """
        Builds our client using the configuration which .
        :return:
        """

        # 1. Load the config whether we're doing file or we've had it passed in as a String.
        if self.config_path:
            self.logger.info("Factory will load configuration from path")
            config_loader = PathConfigLoader(config_path=self.config_path, profile=self.profile)
        elif self.config_json:
            self.logger.info("Factory will load configuration from JSON")
            config_loader = JsonConfigLoader(config_json=self.config_json, profile=self.profile)
        else:
            self.logger.info("Factory will load configuration from home directory")
            config_loader = HomeConfigLoader(profile=self.profile)

        # 2. Parse and validate the client configuration
        try:
            client_config = config_loader.load()
        except Exception as e:
            # Raise a new exception without extra stack trace. We know the JSON failed to parse.
            raise TypeError(f"Unable to build factory due to configuration error: {e}")

        errors = client_config.validate()
        if errors:
            raise TypeError(f"Client configuration is not valid {errors}")

        # 3. Load the validated configuration into our client.
        auth_config = client_config.auth
        auth_type = auth_config.getAuthType()
        logging.info(f"We are processing authentication type [{auth_type}]")

        # Defining the host is optional and defaults to https://api.vectara.io
        # See configuration.py for a list of all supported configuration parameters.
        configuration = vectara_client.Configuration(
            host="https://api.vectara.io"
        )

        if auth_type == "ApiKey":
            auth_util = ApiKeyUtil(client_config.customer_id, client_config.auth.api_key)
            # Configure API key authorization: ApiKeyAuth
            configuration.api_key['ApiKeyAuth'] = client_config.auth.api_key
        elif auth_type == "OAuth2":
            auth_util = OAuthUtil(auth_config.auth_url, auth_config.app_client_id, auth_config.app_client_secret,
                                  client_config.customer_id)
            # TODO: Refactor client to refresh token.
            token = auth_util.getToken()
            configuration.access_token = token
            # configuration.api_key_prefix['ApiKeyAuth'] = 'Bearer'
        else:
            raise TypeError(f"Unknown authentication type: {auth_type}")

        # Uncomment below to setup prefix (e.g. Bearer) for API key, if needed

        # Enter a context with an instance of the API client
        api_client = vectara_client.ApiClient(configuration)

        # Create an instance of the API class
        # api_instance = vectara_client.ApplicationClientsApi(api_client)
        # create_app_client_request = vectara_client.CreateAppClientRequest()  # CreateAppClientRequest |  (optional)

        chats_api = ChatsApi(api_client)
        corpora_api = CorporaApi(api_client)
        documents_api = DocumentsApi(api_client)
        encoders_api = EncodersApi(api_client)
        index_api = IndexApi(api_client)
        jobs_api = JobsApi(api_client)
        llms_api = LargeLanguageModelsApi(api_client)
        queries_api = QueriesApi(api_client)
        rerankers_api = RerankersApi(api_client)
        upload_api = UploadApi(api_client)
        users_api = UsersApi(api_client)



        # try:
        #    # Create an App Client
        #    api_response = api_instance.create_app_client(create_app_client_request=create_app_client_request)
        #    print("The response of ApplicationClientsApi->create_app_client:\n")
        #    pprint(api_response)
        # except Exception as e:
        #    print("Exception when calling ApplicationClientsApi->create_app_client: %s\n" % e)

        # TODO Use the type of authentication to validate whether we can enabled the admin service.
        # corpus_manager = CorpusManager(admin_service, indexer_service)

        return Client(client_config.customer_id, chats_api, corpora_api, documents_api, encoders_api, index_api,
                      jobs_api, llms_api, queries_api, rerankers_api, upload_api, users_api)



# coding: utf-8

"""
    Vectara REST API v2

    Vectara provides an end-to-end platform for creating GenAI products using  a simple to use API.  You can [sign up for an account](https://console.vectara.com/signup) and  then view several [API Recipes](https://docs.vectara.com/docs/api-recipes) with example queries  and parameter values.  The Vectara API Playground lets you experiment with REST endpoints from  your browser. Select an endpoint to view its definition, including the  required or optional headers, body, responses, and sample commands. On the  right side of each endpoint page, like [Get Corpus](/docs/rest-api/get-corpus), you manually  enter your API Key or OAuth Bearer Token, `customer_id`, and then any  required body parameters like the `corpusID` before sending the API  request.  :::note  Vectara supports two primary methods of authentication: API keys and OAuth  2.0, which are applicable to all endpoints. Query API Keys are used for  read-only querying operations, while Index API Keys provide read and write  access. The OAuth 2.0 operations authenticate with a Bearer Token via the  OAuth 2.0 client credentials grant. Review the [**OAuth 2.0 section**](https://docs.vectara.com/docs/learn/authentication/oauth-2) about  how to generate the token. 

    The version of the OpenAPI document: 2.0.0
    Contact: feedback@vectara.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr, field_validator
from typing import Optional
from typing_extensions import Annotated
from vectara_client.models.corpus import Corpus
from vectara_client.models.create_corpus_request import CreateCorpusRequest
from vectara_client.models.list_corpora_response import ListCorporaResponse
from vectara_client.models.replace_filter_attributes_request import ReplaceFilterAttributesRequest
from vectara_client.models.replace_filter_attributes_response import ReplaceFilterAttributesResponse
from vectara_client.models.update_corpus_request import UpdateCorpusRequest

from vectara_client.api_client import ApiClient, RequestSerialized
from vectara_client.api_response import ApiResponse
from vectara_client.rest import RESTResponseType


class CorporaApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_corpus(
        self,
        create_corpus_request: Optional[CreateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Corpus:
        """Create a corpus

        Create a corpus, which is a container to store documents and associated metadata.

        :param create_corpus_request:
        :type create_corpus_request: CreateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_corpus_serialize(
            create_corpus_request=create_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Corpus",
            '400': "BadRequestError",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_corpus_with_http_info(
        self,
        create_corpus_request: Optional[CreateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Corpus]:
        """Create a corpus

        Create a corpus, which is a container to store documents and associated metadata.

        :param create_corpus_request:
        :type create_corpus_request: CreateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_corpus_serialize(
            create_corpus_request=create_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Corpus",
            '400': "BadRequestError",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_corpus_without_preload_content(
        self,
        create_corpus_request: Optional[CreateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a corpus

        Create a corpus, which is a container to store documents and associated metadata.

        :param create_corpus_request:
        :type create_corpus_request: CreateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_corpus_serialize(
            create_corpus_request=create_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Corpus",
            '400': "BadRequestError",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_corpus_serialize(
        self,
        create_corpus_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_corpus_request is not None:
            _body_params = create_corpus_request


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v2/corpora',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_corpus(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to delete")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete a corpus and all its data

        Delete a corpus and all the data that it contains.

        :param corpus_key: The unique key identifying the corpus to delete (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_corpus_with_http_info(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to delete")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete a corpus and all its data

        Delete a corpus and all the data that it contains.

        :param corpus_key: The unique key identifying the corpus to delete (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_corpus_without_preload_content(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to delete")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete a corpus and all its data

        Delete a corpus and all the data that it contains.

        :param corpus_key: The unique key identifying the corpus to delete (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_corpus_serialize(
        self,
        corpus_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if corpus_key is not None:
            _path_params['corpus_key'] = corpus_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/v2/corpora/{corpus_key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_corpus(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Corpus:
        """Retrieve metadata about a corpus

        Get metadata about a corpus. This operation is not a method of searching a corpus.

        :param corpus_key: The unique key identifying the corpus to retrieve. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_corpus_with_http_info(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Corpus]:
        """Retrieve metadata about a corpus

        Get metadata about a corpus. This operation is not a method of searching a corpus.

        :param corpus_key: The unique key identifying the corpus to retrieve. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_corpus_without_preload_content(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve metadata about a corpus

        Get metadata about a corpus. This operation is not a method of searching a corpus.

        :param corpus_key: The unique key identifying the corpus to retrieve. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_corpus_serialize(
        self,
        corpus_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if corpus_key is not None:
            _path_params['corpus_key'] = corpus_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v2/corpora/{corpus_key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_corpora(
        self,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="The maximum number of corpora to return at one time.")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="A regular expression to filter the corpora by their name or summary.")] = None,
        page_key: Annotated[Optional[StrictStr], Field(description="Used to retrieve the next page of corpora after the limit has been reached.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ListCorporaResponse:
        """List corpora

        List corpora in the account. The corpus objects that are returned are less detailed than the direct corpus retrieval operation. 

        :param limit: The maximum number of corpora to return at one time.
        :type limit: int
        :param filter: A regular expression to filter the corpora by their name or summary.
        :type filter: str
        :param page_key: Used to retrieve the next page of corpora after the limit has been reached.
        :type page_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_corpora_serialize(
            limit=limit,
            filter=filter,
            page_key=page_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCorporaResponse",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_corpora_with_http_info(
        self,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="The maximum number of corpora to return at one time.")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="A regular expression to filter the corpora by their name or summary.")] = None,
        page_key: Annotated[Optional[StrictStr], Field(description="Used to retrieve the next page of corpora after the limit has been reached.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ListCorporaResponse]:
        """List corpora

        List corpora in the account. The corpus objects that are returned are less detailed than the direct corpus retrieval operation. 

        :param limit: The maximum number of corpora to return at one time.
        :type limit: int
        :param filter: A regular expression to filter the corpora by their name or summary.
        :type filter: str
        :param page_key: Used to retrieve the next page of corpora after the limit has been reached.
        :type page_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_corpora_serialize(
            limit=limit,
            filter=filter,
            page_key=page_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCorporaResponse",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_corpora_without_preload_content(
        self,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="The maximum number of corpora to return at one time.")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="A regular expression to filter the corpora by their name or summary.")] = None,
        page_key: Annotated[Optional[StrictStr], Field(description="Used to retrieve the next page of corpora after the limit has been reached.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List corpora

        List corpora in the account. The corpus objects that are returned are less detailed than the direct corpus retrieval operation. 

        :param limit: The maximum number of corpora to return at one time.
        :type limit: int
        :param filter: A regular expression to filter the corpora by their name or summary.
        :type filter: str
        :param page_key: Used to retrieve the next page of corpora after the limit has been reached.
        :type page_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_corpora_serialize(
            limit=limit,
            filter=filter,
            page_key=page_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCorporaResponse",
            '403': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_corpora_serialize(
        self,
        limit,
        filter,
        page_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if filter is not None:
            
            _query_params.append(('filter', filter))
            
        if page_key is not None:
            
            _query_params.append(('page_key', page_key))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v2/corpora',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def replace_filter_attributes(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="Key of the corpus to have filters replaced.")],
        replace_filter_attributes_request: Optional[ReplaceFilterAttributesRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ReplaceFilterAttributesResponse:
        """Replace the filter attributes of a corpus

        Replace the filter attributes of a corpus. This does not happen immediately, but instead creates a job and will complete when that job completes. Until that job completes, using new filter attributes will not work.  You can monitor the status of the filter change using the returned job id. 

        :param corpus_key: Key of the corpus to have filters replaced. (required)
        :type corpus_key: str
        :param replace_filter_attributes_request:
        :type replace_filter_attributes_request: ReplaceFilterAttributesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._replace_filter_attributes_serialize(
            corpus_key=corpus_key,
            replace_filter_attributes_request=replace_filter_attributes_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ReplaceFilterAttributesResponse",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def replace_filter_attributes_with_http_info(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="Key of the corpus to have filters replaced.")],
        replace_filter_attributes_request: Optional[ReplaceFilterAttributesRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ReplaceFilterAttributesResponse]:
        """Replace the filter attributes of a corpus

        Replace the filter attributes of a corpus. This does not happen immediately, but instead creates a job and will complete when that job completes. Until that job completes, using new filter attributes will not work.  You can monitor the status of the filter change using the returned job id. 

        :param corpus_key: Key of the corpus to have filters replaced. (required)
        :type corpus_key: str
        :param replace_filter_attributes_request:
        :type replace_filter_attributes_request: ReplaceFilterAttributesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._replace_filter_attributes_serialize(
            corpus_key=corpus_key,
            replace_filter_attributes_request=replace_filter_attributes_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ReplaceFilterAttributesResponse",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def replace_filter_attributes_without_preload_content(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="Key of the corpus to have filters replaced.")],
        replace_filter_attributes_request: Optional[ReplaceFilterAttributesRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Replace the filter attributes of a corpus

        Replace the filter attributes of a corpus. This does not happen immediately, but instead creates a job and will complete when that job completes. Until that job completes, using new filter attributes will not work.  You can monitor the status of the filter change using the returned job id. 

        :param corpus_key: Key of the corpus to have filters replaced. (required)
        :type corpus_key: str
        :param replace_filter_attributes_request:
        :type replace_filter_attributes_request: ReplaceFilterAttributesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._replace_filter_attributes_serialize(
            corpus_key=corpus_key,
            replace_filter_attributes_request=replace_filter_attributes_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ReplaceFilterAttributesResponse",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _replace_filter_attributes_serialize(
        self,
        corpus_key,
        replace_filter_attributes_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if corpus_key is not None:
            _path_params['corpus_key'] = corpus_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if replace_filter_attributes_request is not None:
            _body_params = replace_filter_attributes_request


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v2/corpora/{corpus_key}/replace_filter_attributes',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def reset_corpus(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to reset.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Remove all documents and data in a corpus

        Resets a corpus, which removes all documents and data from the specified corpus, while keeping the corpus itself.

        :param corpus_key: The unique key identifying the corpus to reset. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reset_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def reset_corpus_with_http_info(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to reset.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Remove all documents and data in a corpus

        Resets a corpus, which removes all documents and data from the specified corpus, while keeping the corpus itself.

        :param corpus_key: The unique key identifying the corpus to reset. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reset_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def reset_corpus_without_preload_content(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to reset.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Remove all documents and data in a corpus

        Resets a corpus, which removes all documents and data from the specified corpus, while keeping the corpus itself.

        :param corpus_key: The unique key identifying the corpus to reset. (required)
        :type corpus_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reset_corpus_serialize(
            corpus_key=corpus_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _reset_corpus_serialize(
        self,
        corpus_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if corpus_key is not None:
            _path_params['corpus_key'] = corpus_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v2/corpora/{corpus_key}/reset',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_corpus(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to update.")],
        update_corpus_request: Optional[UpdateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Corpus:
        """Update a corpus

        Enable or disable a corpus.

        :param corpus_key: The unique key identifying the corpus to update. (required)
        :type corpus_key: str
        :param update_corpus_request:
        :type update_corpus_request: UpdateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_corpus_serialize(
            corpus_key=corpus_key,
            update_corpus_request=update_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_corpus_with_http_info(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to update.")],
        update_corpus_request: Optional[UpdateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Corpus]:
        """Update a corpus

        Enable or disable a corpus.

        :param corpus_key: The unique key identifying the corpus to update. (required)
        :type corpus_key: str
        :param update_corpus_request:
        :type update_corpus_request: UpdateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_corpus_serialize(
            corpus_key=corpus_key,
            update_corpus_request=update_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_corpus_without_preload_content(
        self,
        corpus_key: Annotated[str, Field(strict=True, max_length=50, description="The unique key identifying the corpus to update.")],
        update_corpus_request: Optional[UpdateCorpusRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update a corpus

        Enable or disable a corpus.

        :param corpus_key: The unique key identifying the corpus to update. (required)
        :type corpus_key: str
        :param update_corpus_request:
        :type update_corpus_request: UpdateCorpusRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_corpus_serialize(
            corpus_key=corpus_key,
            update_corpus_request=update_corpus_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Corpus",
            '403': "Error",
            '404': "NotFoundError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_corpus_serialize(
        self,
        corpus_key,
        update_corpus_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if corpus_key is not None:
            _path_params['corpus_key'] = corpus_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_corpus_request is not None:
            _body_params = update_corpus_request


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/v2/corpora/{corpus_key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





import re
from urllib.parse import urlparse
import unittest
import requests
import logging
from authlib.integrations.requests_client import OAuth2Session
import datetime
import traceback
from abc import ABC

# TODO This should be discovered from the console
# TODO Change this file to authc.py!!
DEFAULT_OAUTH2_URL = "https://vectara-prod-{customer_id}.auth.us-west-2.amazoncognito.com/oauth2/token"

logger = logging.getLogger(__name__)


class BaseAuthUtil(ABC):

    def get_headers(self) -> dict:
        raise NotImplementedError("You must implement this on a subclass")

    def authenticate(self):
        pass

class OAuthUtil(BaseAuthUtil):

    def __init__(self, auth_url: str, app_client_id: str, app_client_secret: str, customer_id: str):
        self.logger = logging.getLogger(str(__class__.__name__))

        if not customer_id:
            # Putting this in as I'm surprised we also need a customer Id on the header.
            raise TypeError("You must include a customer Id, apparently?")
        else:
            self.customer_id = customer_id

        # Validate
        if not auth_url and not customer_id:
            raise TypeError("You must either put in a client Id or OAuth authentication URL")
        if not app_client_secret:
            raise TypeError("You must include an application client secret")
        if not app_client_id:
            raise TypeError("You must include an application client Id")



        if auth_url:
            self.logger.info(f"Using provided OAuth2 URL [{auth_url}]")
            self.auth_url = auth_url
        else:
            self.logger.info(f"No Authentication URL Provided for OAuth2, creating default from customer id [{customer_id}]")
            self.auth_url = (DEFAULT_OAUTH2_URL).format(customer_id=customer_id)
        self.logger.info(f"OAuth2 URL is [{self.auth_url}]")

        self.app_client_id = app_client_id
        if self.app_client_id:
            self.logger.debug(f"OAuth Application Client Id: {self.app_client_id}")
        self.app_client_secret = app_client_secret
        if self.app_client_secret:
            self.logger.debug(f"OAuth Application Client Secret: {self.app_client_secret}")

        self.expiry_ts = None
        self.expiry_txt = None
        self.expires_in = None
        self.expires_at = None
        self.access_token = None

    def authenticate(self):
        """Connect to the server and get a JWT token."""

#        try:
        session = OAuth2Session(
            self.app_client_id, self.app_client_secret, scope="")
        token = session.fetch_token(self.auth_url, grant_type="client_credentials")
        self.expires_at = token["expires_at"]
        self.expires_in = token["expires_in"]
        self.access_token = token["access_token"]

        self.expiry_ts = datetime.datetime.fromtimestamp(self.expires_at)
        self.expiry_txt = self.expiry_ts.strftime("%m/%d/%Y, %H:%M:%S")
        self.logger.info(f"Received OAuth token, will expire [{self.expiry_txt}]")

    def getToken(self):
        """Get the current token or get a new one if expired"""
        now = datetime.datetime.now()
        self.logger.info(f"Current timestamp {now}")

        if self.expiry_ts:
            self.logger.info(f"Expiry            {self.expiry_ts}")

        if not self.access_token:
            self.logger.info("First time requesting token, authenticating")
            self.authenticate()
        if (now > (self.expiry_ts - datetime.timedelta(seconds=5))):
            self.logger.info("Token expiry within 5 seconds, refreshing")
            self.authenticate()
        else:
            self.logger.info(f"Already authenticated with non-expired token, expiry is [{self.expires_at}]")

        return self.access_token

    def get_headers(self) -> dict:
        token = self.getToken()

        return {
            "Customer-Id": self.customer_id,
            "Authorization": f"Bearer {token}"
        }

class ApiKeyUtil(BaseAuthUtil):

    def __init__(self, customer_id, api_key):
        self.customer_id = customer_id
        self.api_key = api_key

    def get_headers(self):
        return {
            "customer-id": self.customer_id,
            "x-api-key": self.api_key
        }